use crate::config::Config;
use crate::indexes::{SerializableMessage, SerializableThread};
use serde::{Deserialize, Serialize};
use std::collections::HashMap;
use std::fs::File;
use std::io;
use std::io::{BufReader, Read};
use chrono::NaiveDate;

pub async fn get_folders() -> String {
    let mut folders: Vec<String> = Vec::new();
    for entry in std::fs::read_dir(Config::global().out_dir.clone())
        .expect("could not read maildir")
        .filter_map(|m| m.ok())
    {
        let metadata = match entry.metadata() {
            Ok(metadata) => metadata,
            Err(_) => continue,
        };
        if !metadata.is_dir() {
            continue;
        }
        folders.push(entry.file_name().into_string().unwrap());
    }

    serde_json::to_string(&folders).unwrap()
}

pub async fn get_email(id: String) -> String {
    let messages_path = Config::global()
        .out_dir
        .clone()
        .join("messages.json");
    let messages_file = match File::open(messages_path) {
        Ok(file) => file,
        Err(_) => return String::new(),
    };
    let messages_reader = BufReader::new(messages_file);
    let messages: Vec<SerializableMessage> = match serde_json::from_reader(messages_reader)
    {
        Ok(messages) => messages,
        Err(_) => return String::new(),
    };
    let message = match messages.iter().find(|message| message.id == id){
        None => {return String::new()}
        Some(message) => {message}
    };
    
    let abs_path = Config::global().out_dir.clone()
        .join(message.list.clone())
        .join("messages")
        .join(message.hash.clone() + ".html");
    let mut file = match File::open(abs_path.clone()) {
        Ok(content) => {content}
        Err(_) => {return "".to_string()}
    };
    let mut content = String::new();
    match file.read_to_string(&mut content) {
        Ok(_) => {}
        Err(_) => {return String::new()}
    };
    content
}

pub async fn get_thread_messages(
    id: u32
) -> String {
    let threads_path = Config::global()
        .out_dir
        .clone()
        .join("threads.json");
    let messages_path = Config::global()
        .out_dir
        .clone()
        .join("messages.json");
    
    let threads_file = match File::open(threads_path) {
        Ok(file) => file,
        Err(_) => return serde_json::to_string(&Vec::<String>::new()).unwrap(),
    };
    let messages_file = match File::open(messages_path) {
        Ok(file) => file,
        Err(_) => return serde_json::to_string(&Vec::<SerializableMessage>::new()).unwrap(),
    };
    
    let threads_reader = BufReader::new(threads_file);
    let messages_reader = BufReader::new(messages_file);
    
    let threads: Vec<SerializableThread> = match serde_json::from_reader(threads_reader)
    {
        Ok(threads) => threads,
        Err(_) => return serde_json::to_string(&Vec::<SerializableMessage>::new()).unwrap(),
    };

    if let Some(thread) = threads.into_iter().find(|thread| thread.id == id) {
        let messages: Vec<SerializableMessage> = match serde_json::from_reader(messages_reader)
        {
            Ok(messages) => messages,
            Err(_) => return serde_json::to_string(&Vec::<SerializableMessage>::new()).unwrap(),
        };

        let result_messages: Vec<SerializableMessage> = thread.messages.into_iter()
            .filter_map(|message_id| {
                messages.iter().find(|message| message.id == message_id).cloned()
            })
            .collect();

        serde_json::to_string(&result_messages).unwrap()
    } else {
        serde_json::to_string(&Vec::<SerializableMessage>::new()).unwrap()
    }
}

async fn get_threads(
    folder: String,
    file_name: String,
) -> String {
    let path = Config::global()
        .out_dir
        .clone()
        .join(folder)
        .join(file_name);

    let file = match File::open(path) {
        Ok(file) => file,
        Err(_) => return serde_json::to_string(&Vec::<(String, Vec<u32>)>::new()).unwrap(),
    };
    let reader = BufReader::new(file);
    let messages: Vec<(String, Vec<u32>)> = match serde_json::from_reader(reader)
    {
        Ok(messages) => messages,
        Err(_) => return serde_json::to_string(&Vec::<(String, Vec<u32>)>::new()).unwrap(),
    };

    serde_json::to_string(&messages).unwrap()
}

pub async fn get_sorted_threads_by_date(
    folder: String
) -> String {
    get_threads(folder, "date.json".to_string()).await
}
